@@ -3,5 +3,8 @@
 mod distance;
 pub use self::distance::*;
 
+mod remedian;
+pub use self::remedian::Remedian;
+
 mod statistics;
 pub use self::statistics::*;
