@@ -4,6 +4,7 @@
 #[path = "../tests/unit/example_test.rs"]
 mod example_test;
 
+use crate::algorithms::gsom::Input;
 use crate::evolution::*;
 use crate::hyper::*;
 use crate::population::{DominanceOrder, DominanceOrdered, RosomaxaWeighted, Shuffled};
@@ -41,6 +42,7 @@ pub struct VectorObjective {
 pub struct VectorSolution {
     /// Solution payload.
     pub data: Vec<f64>,
+    weights: Vec<f64>,
     objective: Arc<VectorObjective>,
     order: DominanceOrder,
 }
@@ -171,15 +173,21 @@ impl DominanceOrdered for VectorSolution {
 }
 
 impl RosomaxaWeighted for VectorSolution {
-    fn weights(&self) -> Vec<f64> {
-        self.objective.weight_fn.deref()(self.data.as_slice())
+    fn init_weights(&mut self) {
+        self.weights = self.objective.weight_fn.deref()(self.data.as_slice());
+    }
+}
+
+impl Input for VectorSolution {
+    fn weights(&self) -> &[f64] {
+        self.weights.as_slice()
     }
 }
 
 impl VectorSolution {
     /// Creates a new instance of `VectorSolution`.
     pub fn new(data: Vec<f64>, objective: Arc<VectorObjective>) -> Self {
-        Self { data, objective, order: DominanceOrder::default() }
+        Self { data, objective, weights: Vec::default(), order: DominanceOrder::default() }
     }
 }
 
@@ -269,7 +277,7 @@ pub type ContextFactory = Box<dyn FnOnce(Arc<VectorObjective>, Arc<Environment>)
 /// An example of the optimization solver to solve trivial problems.
 pub struct Solver {
     logger: Option<InfoLogger>,
-    use_dynamic_heuristic_only: bool,
+    use_static_heuristic: bool,
     initial_solutions: Vec<Vec<f64>>,
     initial_params: (usize, f64),
     fitness_fn: Option<FitnessFn>,
@@ -286,7 +294,7 @@ impl Default for Solver {
     fn default() -> Self {
         Self {
             logger: None,
-            use_dynamic_heuristic_only: false,
+            use_static_heuristic: false,
             initial_solutions: vec![],
             initial_params: (4, 0.05),
             fitness_fn: None,
@@ -303,8 +311,8 @@ impl Default for Solver {
 
 impl Solver {
     /// Use dynamic selective only
-    pub fn use_dynamic_heuristic_only(mut self) -> Self {
-        self.use_dynamic_heuristic_only = true;
+    pub fn use_static_heuristic(mut self) -> Self {
+        self.use_static_heuristic = true;
         self
     }
 
@@ -379,13 +387,10 @@ impl Solver {
         });
 
         // build instances of implementation types from submitted data
-        let heuristic = if self.use_dynamic_heuristic_only {
-            self.create_dynamic_heuristic(environment.clone())
+        let heuristic = if self.use_static_heuristic {
+            self.create_static_heuristic(environment.clone())
         } else {
-            Box::new(MultiSelective::new(
-                self.create_dynamic_heuristic(environment.clone()),
-                self.create_static_heuristic(environment.clone()),
-            ))
+            self.create_dynamic_heuristic(environment.clone())
         };
         let fitness_fn = self.fitness_fn.ok_or_else(|| "objective function must be set".to_string())?;
         let weight_fn = self.weight_fn.unwrap_or_else({
