@@ -53,10 +53,10 @@ impl RosomaxaConfig {
     }
 }
 
-/// Specifies behavior which returns a weights used to distinguish different solutions.
-pub trait RosomaxaWeighted {
-    /// Returns a weights used to distinguish different solutions.
-    fn weights(&self) -> Vec<f64>;
+/// Specifies behavior which keeps track of weights used to distinguish different solutions.
+pub trait RosomaxaWeighted: Input {
+    /// Initializes weights.
+    fn init_weights(&mut self);
 }
 
 /// Implements custom algorithm, code name Routing Optimizations with Self Organizing
@@ -87,7 +87,7 @@ where
         let elite = individuals
             .iter()
             .filter(|individual| self.is_comparable_with_best_known(individual, best_known))
-            .map(|individual| individual.deep_copy())
+            .map(|individual| init_individual(individual.deep_copy()))
             .collect::<Vec<_>>();
         let is_improved = self.elite.add_all(elite);
 
@@ -96,7 +96,7 @@ where
                 known_individuals.extend(individuals.into_iter())
             }
             RosomaxaPhases::Exploration { network, statistics, .. } => {
-                network.store_batch(individuals, statistics.generation, IndividualInput::new);
+                network.store_batch(individuals, statistics.generation, init_individual);
             }
             RosomaxaPhases::Exploitation { .. } => {}
         }
@@ -106,6 +106,7 @@ where
 
     fn add(&mut self, individual: Self::Individual) -> bool {
         let best_known = self.elite.ranked().map(|(i, _)| i).next();
+        let individual = init_individual(individual);
         let is_improved = if self.is_comparable_with_best_known(&individual, best_known) {
             self.elite.add(individual.deep_copy())
         } else {
@@ -114,9 +115,7 @@ where
 
         match &mut self.phase {
             RosomaxaPhases::Initial { solutions: individuals } => individuals.push(individual),
-            RosomaxaPhases::Exploration { network, statistics, .. } => {
-                network.store(IndividualInput::new(individual), statistics.generation)
-            }
+            RosomaxaPhases::Exploration { network, statistics, .. } => network.store(individual, statistics.generation),
             RosomaxaPhases::Exploitation { .. } => {}
         }
 
@@ -147,7 +146,7 @@ where
                     self.elite
                         .select()
                         .take(elite_explore_size)
-                        .chain(coordinates.iter().flat_map(move |(coordinate, _, _)| {
+                        .chain(coordinates.iter().flat_map(move |coordinate| {
                             let explore_size = self.environment.random.uniform_int(1, node_explore_size) as usize;
 
                             network
@@ -202,7 +201,7 @@ where
     }
 }
 
-type IndividualNetwork<O, S> = Network<IndividualInput<S>, IndividualStorage<O, S>, IndividualStorageFactory<O, S>>;
+type IndividualNetwork<O, S> = Network<S, IndividualStorage<O, S>, IndividualStorageFactory<O, S>>;
 
 impl<O, S> Rosomaxa<O, S>
 where
@@ -218,7 +217,13 @@ where
         Ok(Self {
             objective: objective.clone(),
             environment: environment.clone(),
-            elite: Elitism::new(objective, environment.random.clone(), config.elite_size, config.selection_size),
+            elite: Elitism::new_with_dedup(
+                objective,
+                environment.random.clone(),
+                config.elite_size,
+                config.selection_size,
+                create_dedup_fn(0.02),
+            ),
             phase: RosomaxaPhases::Initial { solutions: vec![] },
             config,
         })
@@ -239,7 +244,7 @@ where
                         &self.config,
                         individuals.drain(0..4).collect(),
                     );
-                    individuals.drain(0..).for_each(|individual| network.store(IndividualInput::new(individual), 0));
+                    individuals.drain(0..).for_each(|individual| network.store(init_individual(individual), 0));
 
                     self.phase = RosomaxaPhases::Exploration {
                         network,
@@ -275,12 +280,7 @@ where
                         self.config.learning_rate,
                     );
 
-                    Self::fill_populations(
-                        network,
-                        coordinates,
-                        best_fitness.as_slice(),
-                        self.environment.random.as_ref(),
-                    );
+                    Self::fill_populations(network, coordinates, self.environment.random.as_ref());
                 } else {
                     self.phase = RosomaxaPhases::Exploitation { selection_size }
                 }
@@ -297,22 +297,17 @@ where
 
     fn fill_populations<'a>(
         network: &'a IndividualNetwork<O, S>,
-        coordinates: &mut Vec<(Coordinate, f64, usize)>,
-        best_fitness: &[f64],
+        coordinates: &mut Vec<Coordinate>,
         random: &(dyn Random + Send + Sync),
     ) {
         coordinates.clear();
         coordinates.extend(network.iter().filter_map(|(coordinate, node)| {
             let node = node.read().unwrap();
-            let coordinate = node.storage.population.select().next().map(|individual| {
-                (
-                    *coordinate,
-                    relative_distance(best_fitness.iter().cloned(), individual.get_fitness()),
-                    node.get_last_hits(network.get_current_time()),
-                )
-            });
-
-            coordinate
+            if node.storage.population.size() > 0 {
+                Some(*coordinate)
+            } else {
+                None
+            }
         }));
 
         coordinates.shuffle(&mut random.get_rng());
@@ -347,7 +342,13 @@ where
             network.set_learning_rate(statistics.termination_estimate.clamp(init_learning_rate, 1.));
         }
 
-        let get_distance = |node: &NodeLink<IndividualInput<S>, IndividualStorage<O, S>>| {
+        let max_unified_distance = network
+            .get_nodes()
+            .map(|node| node.read().unwrap().unified_distance(network, 1))
+            .max_by(|a, b| compare_floats(*a, *b))
+            .unwrap_or(0.);
+
+        let get_distance = |node: &NodeLink<S, IndividualStorage<O, S>>| {
             let node = node.read().unwrap();
             let individual = node.storage.population.ranked().next();
 
@@ -366,7 +367,9 @@ where
                 // NOTE
                 // unified distance filter improves diversity property
                 // distance filter improves exploitation characteristic by removing old (or empty) nodes
-                unified_distance > 0.1 && get_distance(node).map_or(false, |distance| distance < distance_threshold)
+
+                let is_far_enough = compare_floats(unified_distance, max_unified_distance * 0.1) != Ordering::Less;
+                is_far_enough && get_distance(node).map_or(false, |distance| distance < distance_threshold)
             });
             network.smooth(1);
         }
@@ -378,10 +381,10 @@ where
         config: &RosomaxaConfig,
         individuals: Vec<S>,
     ) -> IndividualNetwork<O, S> {
-        let inputs_vec = individuals.into_iter().map(IndividualInput::new).collect::<Vec<_>>();
+        let inputs_vec = individuals.into_iter().map(init_individual).collect::<Vec<_>>();
 
         let inputs_slice = inputs_vec.into_boxed_slice();
-        let inputs_array: Box<[IndividualInput<S>; 4]> = match inputs_slice.try_into() {
+        let inputs_array: Box<[S; 4]> = match inputs_slice.try_into() {
             Ok(ba) => ba,
             Err(o) => panic!("expected individuals of length {} but it was {}", 4, o.len()),
         };
@@ -450,7 +453,7 @@ where
     },
     Exploration {
         network: IndividualNetwork<O, S>,
-        coordinates: Vec<(Coordinate, f64, usize)>,
+        coordinates: Vec<Coordinate>,
         statistics: HeuristicStatistics,
         selection_size: usize,
     },
@@ -459,30 +462,14 @@ where
     },
 }
 
-struct IndividualInput<S>
+fn init_individual<S>(individual: S) -> S
 where
     S: HeuristicSolution + RosomaxaWeighted + DominanceOrdered,
 {
-    weights: Vec<f64>,
-    individual: S,
-}
+    let mut individual = individual;
+    individual.init_weights();
 
-impl<S> IndividualInput<S>
-where
-    S: HeuristicSolution + RosomaxaWeighted + DominanceOrdered,
-{
-    pub fn new(individual: S) -> Self {
-        Self { weights: individual.weights(), individual }
-    }
-}
-
-impl<S> Input for IndividualInput<S>
-where
-    S: HeuristicSolution + RosomaxaWeighted + DominanceOrdered,
-{
-    fn weights(&self) -> &[f64] {
-        self.weights.as_slice()
-    }
+    individual
 }
 
 struct IndividualStorageFactory<O, S>
@@ -496,7 +483,7 @@ where
     objective: Arc<O>,
 }
 
-impl<O, S> StorageFactory<IndividualInput<S>, IndividualStorage<O, S>> for IndividualStorageFactory<O, S>
+impl<O, S> StorageFactory<S, IndividualStorage<O, S>> for IndividualStorageFactory<O, S>
 where
     O: HeuristicObjective<Solution = S> + Shuffled,
     S: HeuristicSolution + RosomaxaWeighted + DominanceOrdered,
@@ -507,16 +494,7 @@ where
             self.random.clone(),
             self.node_size,
             self.node_size,
-            Box::new(|a, b| {
-                // TODO return slice instead of vector?
-                let weights_a = a.weights();
-                let weights_b = b.weights();
-
-                let distance = relative_distance(weights_a.iter().cloned(), weights_b.iter().cloned());
-
-                // NOTE custom dedup rule to increase diversity property
-                distance < 0.1
-            }),
+            create_dedup_fn(0.1),
         );
         if self.random.is_hit(self.reshuffling_probability) {
             elitism.shuffle_objective();
@@ -538,17 +516,17 @@ where
     O: HeuristicObjective<Solution = S> + Shuffled,
     S: HeuristicSolution + RosomaxaWeighted + DominanceOrdered,
 {
-    type Item = IndividualInput<S>;
+    type Item = S;
 
     fn add(&mut self, input: Self::Item) {
-        self.population.add(input.individual);
+        self.population.add(input);
     }
 
     fn drain<R>(&mut self, range: R) -> Vec<Self::Item>
     where
         R: RangeBounds<usize>,
     {
-        self.population.drain(range).into_iter().map(IndividualInput::new).collect()
+        self.population.drain(range).into_iter().collect()
     }
 
     fn distance(&self, a: &[f64], b: &[f64]) -> f64 {
@@ -569,3 +547,26 @@ where
         write!(f, "{}", self.population)
     }
 }
+
+fn create_dedup_fn<O, S>(threshold: f64) -> Box<dyn Fn(&O, &S, &S) -> bool + Send + Sync>
+where
+    O: HeuristicObjective<Solution = S> + Shuffled,
+    S: HeuristicSolution + RosomaxaWeighted + DominanceOrdered,
+{
+    // NOTE custom dedup rule to increase diversity property
+    Box::new(move |objective, a, b| match objective.total_order(a, b) {
+        Ordering::Equal => {
+            let fitness_a = a.get_fitness();
+            let fitness_b = b.get_fitness();
+
+            fitness_a.zip(fitness_b).all(|(a, b)| compare_floats(a, b) == Ordering::Equal)
+        }
+        _ => {
+            let weights_a = a.weights();
+            let weights_b = b.weights();
+            let distance = relative_distance(weights_a.iter().cloned(), weights_b.iter().cloned());
+
+            distance < threshold
+        }
+    })
+}
